@@ -1,21 +1,8 @@
 import argparse
-import sys
 import unittest
-from unittest.mock import patch
 
 from chromie.parser import parse_args
-
-
-class ParserHelper:
-    @classmethod
-    def get_mocked_args(self, cmd):
-        argv = self.get_argv(cmd)
-        with patch.object(sys, "argv", argv):
-            return parse_args(argv=sys.argv)
-
-    @classmethod
-    def get_argv(self, cmd):
-        return cmd.split()[1:]
+from tests.utils import ParserHelper
 
 
 class TestArgumentParser(unittest.TestCase):
